import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_interfaces import user_pb2 as User


class CoreApi:
    def __init__(self, env, auth: AuthenticationApi):
        self.AppUrl = "/enterprise/core/v1/"
        self.Environment = env
        self.Authentication = auth

    def GetUser(self, userId, expand=None):
        user = User.User()
        if (expand != None):
            url = self.Environment+self.AppUrl+"User/"+userId+"expand="+expand
        else:
            url = self.Environment+self.AppUrl+"User/"+userId
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.users.items[0]


class UserHelper:
    def GetUserFromUserInfo(userInfo):
        jResponse = json.loads(userInfo.content)



import csv
import json
import logging
from one_py_sdk.operations.spreadsheet import SpreadsheetApi
from one_py_sdk.common.library import LibraryApi
from one_py_sdk.enterprise.twin import DigitalTwinApi
from one_py_sdk.common.configuration import ConfigurationApi
from one_py_sdk.historian.data import HistorianApi
from one_py_sdk.shared.helpers.datetimehelper import *
from datetime import datetime, timezone


class Exporter:
    def __init__(self, env, auth):
        self.Environment = env
        self.Authentication = auth
        self.Spreadsheet = SpreadsheetApi(env, auth)
        self.Library = LibraryApi(env, auth)
        self.DigitalTwin = DigitalTwinApi(env, auth)
        self.Configuration = ConfigurationApi(env, auth)
        self.Historian = HistorianApi(env, auth)
    dataFieldNames = ['Worksheet Type', 'Time', 'ColumnName', 'ColumnId',
                      'RowNumber', 'Value', 'StringValue', 'DateEntered', 'ChangedUsing']
    columnFieldNames = ['Worksheet Type', 'ColumnNumber', 'Name', 'ParameterId', 'LocationId', 'LocationName', 'LocationType', 'LocationSubtype', 'Path', 'Latitude', 'Longitude',
                        'ParameterTranslation', 'ColumnId', 'UnitId', 'UnitTranslation', 'LastPopulatedDate', 'DataBinding',
                        'LimitName', "LowValue", "LowOperation", "HighValue", "HighOperation", "LimitStartTime", "LimitEndTime"]
    historianDataFieldNames = ["DateUTC", "Value", "String Value",
                               'Property Bag', "CreatedById", "CreatedOn", "ModifiedById", "ModifiedOn", ]
    limitFieldNames = ["ColumnId", "ColumnName", 'LimitName', "LimitType", "LowValue", "LowOperation",
                       "HighValue", "HighOperation", "LimitStartTime", "LimitEndTime", "NotificationsEnabled"]

    def ExportHistorianTwin(self, filename, twinId, startDate, endDate):
        fieldnames = self.historianDataFieldNames
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            historianData = self.Historian.GetHistorianDataRange(
                twinId, startDate, endDate)
            for datum in historianData:
                worksheetWriter.writerow({"DateUTC": datum.dateTimeUTC.jsonDateTime.value,
                                          "Value": datum.value.value,
                                          "String Value": datum.stringValue.value,
                                          'Property Bag': datum.propertyBag.value,
                                          "CreatedById": datum.recordAuditInfo.createdById,
                                          "CreatedOn": datum.recordAuditInfo.createdOn.jsonDateTime.value,
                                          "ModifiedById": datum.recordAuditInfo.modifiedById,
                                          "ModifiedOn": datum.recordAuditInfo.modifiedOn.jsonDateTime.value})

    def ExportWorksheet(self, filename, plantId, startDate, endDate, updatedAfter=None, wsType=None):
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.dataFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1, 5)
                for wsType in wsTypes:
                    try:
                        self.__mapAndWriteRowsAndColumns(
                            worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)
                    except:
                        logging.error(f"Export for ws type {self.ConvertWSTypeToStringValue(wsType)} ")
                        continue
            else:
                self.__mapAndWriteRowsAndColumns(
                    worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)

    def __mapAndWriteRowsAndColumns(self, worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        if not startDate.tzinfo:
            startDate = startDate.replace(tzinfo=timezone.utc)
        if not endDate.tzinfo:
            endDate = endDate.replace(tzinfo=timezone.utc)
        try:
            ws = self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]
        except:           
            return
        if not ws.columns:            
            return
        rows = self.Spreadsheet.GetRowsForTimeRange(
            plantId, wsType, startDate, endDate)
        try:
            rowNumbers = rows.keys()
            rowValues = rows.values()
        except(AttributeError):
            return
        rowDict = {}
        for num in rowNumbers:
            rowDict[num] = str(GetDateFromRowNumber(num, ws.enumWorksheet))

        numberMapping = {}
        for column in ws.columns:
            numberMapping[column.columnNumber] = [column.name, column.columnId]
        for vals in rows.values():
            for cell in vals.cells:
                if updatedAfter != None:
                    try:
                        dateEntered = cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value
                        if dateEntered:
                            dateEntered = self.ParseAuditTime(dateEntered)
                            if not updatedAfter.tzinfo:
                                updatedAfter = updatedAfter.replace(
                                    tzinfo=timezone.utc)
                            if dateEntered > updatedAfter:
                                worksheetWriter.writerow({'Worksheet Type': wsVal,
                                                        'Time': rowDict[vals.rowNumber], 'ColumnName': numberMapping[cell.columnNumber][0],
                                                        'ColumnId': numberMapping[cell.columnNumber][1],
                                                        'Value': cell.cellDatas[0].value.value,
                                                        'RowNumber': vals.rowNumber,
                                                        'StringValue': cell.cellDatas[0].stringValue.value,
                                                        'DateEntered': cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value,
                                                        'ChangedUsing': self.EnumDataSourceToStringValue(cell.cellDatas[0].auditEvents[-1].enumDataSource)})
                        else: 
                            logging.error(f"Audit info not found for Plant: {plantId},'Worksheet Type': {wsVal}, 'ColumnName':{numberMapping[cell.columnNumber][0]},'Time': {rowDict[vals.rowNumber]}, 'Value': {cell.cellDatas[0].value.value} ")
                    except IndexError:
                        pass
                    except TypeError:
                        logging.exception("message")
                        logging.error(
                            f"Input date could not be parsed for'Worksheet Type': {wsVal}, 'ColumnName':{numberMapping[cell.columnNumber][0]},'Time': {rowDict[vals.rowNumber]}, 'Value': {cell.cellDatas[0].value.value}, 'DateEntered':{cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value} ")
                        logging.debug(len(dateEntered))
                    except ValueError:
                        logging.exception("message")
                        logging.error(
                            f"Input date could not be parsed for'Worksheet Type': {wsVal}, 'ColumnName':{numberMapping[cell.columnNumber][0]},'Time': {rowDict[vals.rowNumber]}, 'Value': {cell.cellDatas[0].value.value}, 'DateEntered':{cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value} ")
                        logging.debug(len(dateEntered))
                    except:
                        logging.exception("message")
                        logging.error(
                            f"Input date could not be parsed for'Worksheet Type': {wsVal}, 'ColumnName':{numberMapping[cell.columnNumber][0]},'Time': {rowDict[vals.rowNumber]}, 'Value': {cell.cellDatas[0].value.value}, 'DateEntered':{cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value} ")
                else:
                    try:
                        worksheetWriter.writerow({'Worksheet Type': wsVal,
                                                  'Time': rowDict[vals.rowNumber],
                                                  'ColumnName': numberMapping[cell.columnNumber][0],
                                                  'ColumnId': numberMapping[cell.columnNumber][1],
                                                  'Value': (cell.cellDatas[0].value.value),
                                                  'RowNumber': vals.rowNumber,
                                                  'StringValue': cell.cellDatas[0].stringValue.value,
                                                  'DateEntered': cell.cellDatas[0].auditEvents[-1].timeStamp.jsonDateTime.value,
                                                  'ChangedUsing': self.EnumDataSourceToStringValue(cell.cellDatas[0].auditEvents[-1].enumDataSource)})
                    except(IndexError):
                        pass

    def ExportWorksheetByType(self, filename, plantId, startDate, endDate, wsType=4, updatedAfter=None):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.dataFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            self.__mapAndWriteRowsAndColumns(
                worksheetWriter, plantId, wsType, startDate, endDate, updatedAfter)

    def ExportColumnDetails(self, filename, plantId, wsType=None, viewName=""):
        unitDict, paramDict, typeDict, subtypeDict = self.__mapUnitsAndParams()

        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.columnFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1, 5)
                for wsType in wsTypes:
                    try:
                        self.__mapAndWriteColumns(
                            plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, viewName)
                    except:
                        raise
                        continue
            else:
                self.__mapAndWriteColumns(
                    plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, viewName)

    def ExportLimitColumns(self, filename, plantId, wsType=None, viewName="", allLimits=False):
        unitDict, paramDict, typeDict, subtypeDict = self.__mapUnitsAndParams()
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.columnFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1, 5)
                for wsType in wsTypes:
                    try:
                        self.__mapAndWriteLimitColumns(
                            plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, allLimits, viewName="")
                    except:
                        continue
            else:
                self.__mapAndWriteLimitColumns(
                    plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, allLimits, viewName="")

    def ExportLimits(self, filename, plantId, wsType=None):
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.limitFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            if not wsType:
                wsTypes = range(1, 5)
                for wsType in wsTypes:
                    wsVal = self.ConvertWSTypeToStringValue(wsType)
            try:
                ws = self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[
                    0]
            except:
                return
            if not ws.columns:
                return
            columnDict = {}
            for column in ws.columns:
                if column.limits:
                    columnDict[column.columnId] = [column.name, column.limits]

            for key in columnDict.keys():
                for lim in columnDict[key][1]:
                    worksheetWriter.writerow({
                        'ColumnId': key,
                        'ColumnName': columnDict[key][0],
                        'LimitName': lim.name,
                        'LimitType': self.ConvertLimitEnumToStringValue(lim.enumLimit),
                        'LowValue': lim.lowValue.value,
                        "LowOperation": self.LimitOperationToStringValue(lim.lowOperation),
                        "HighValue": lim.highValue.value,
                        "HighOperation": self.LimitOperationToStringValue(lim.highOperation),
                        "LimitStartTime": lim.timeWindow.startTime.jsonDateTime.value,
                        "LimitEndTime": lim.timeWindow.endTime.jsonDateTime.value,
                        "NotificationsEnabled": lim.notificationFlag
                    })

    def __mapAndWriteLimitColumns(self, plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, allLimits, viewName):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        try:
            ws = self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]
        except:
            return
        if not ws.columns:
            return

        columnDict = {}
        for column in ws.columns:
            columnDict[column.columnNumber] = [column.name,  column.columnId, column.parameterId,  column.displayUnitId,
                                               column.lastRowNumberWithData, column.locationId, column.limits, column.dataSourceBinding]
        limitDict = {}
        for column in columnDict.values():
            try:
                limit = column[6][0]
                if allLimits:
                    limitDict[column[1]] = [limit.name, limit.lowValue.value, limit.lowOperation, limit.highValue.value,
                                            limit.highOperation,  limit.timeWindow.startTime.jsonDateTime.value, limit.timeWindow.endTime.jsonDateTime.value]
                else:
                    if limit.enumLimit == 5 or limit.enumLimit == 4 or limit.enumLimit == 1:
                        limitDict[column[1]] = [limit.name, limit.lowValue.value, limit.lowOperation, limit.highValue.value,
                                                limit.highOperation,  limit.timeWindow.startTime.jsonDateTime.value, limit.timeWindow.endTime.jsonDateTime.value]

            except:
                pass

        twinDict = self.PathFinder(plantId, columnDict)
        for key in columnDict.keys():
            try:
                try:
                    dataSourceBinding = columnDict[key][7]
                except:
                    dataSourceBinding = ""
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                locationType = typeDict[twinDict[columnDict[key][5]][4]][1]
                lat = twinDict[columnDict[key][1]][6]
                long = twinDict[columnDict[key][1]][7]
                worksheetWriter.writerow({'ColumnNumber': key,
                                          'Name': columnDict[key][0],
                                          'ColumnId': columnDict[key][1],
                                          'Worksheet Type': wsVal,
                                          'ParameterId': columnDict[key][2],
                                          'UnitId': columnDict[key][3],
                                          'LocationName': twinDict[columnDict[key][1]][3],
                                          'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                          'UnitTranslation': unitDict[columnDict[key][3]][2],
                                          'LocationId': columnDict[key][5],
                                          'LocationType': locationType,
                                          'ParameterTranslation': paramDict[columnDict[key][2]][1],
                                          "Path": twinDict[columnDict[key][1]][2],
                                          "Latitude": lat,
                                          "Longitude": long,
                                          'LimitName':  limitDict[columnDict[key][1]][0],
                                          "LowValue": limitDict[columnDict[key][1]][1],
                                          "LowOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][2]),
                                          "HighValue": limitDict[columnDict[key][1]][3],
                                          "HighOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][4]),
                                          "LimitStartTime": limitDict[columnDict[key][1]][5],
                                          "LimitEndTime": limitDict[columnDict[key][1]][6],
                                          "LocationSubtype": locationSubtype,
                                          "DataBinding": dataSourceBinding

                                          })
            except (KeyError):
                pass

    def __mapAndWriteColumns(self, plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, viewName=""):
        wsVal = self.ConvertWSTypeToStringValue(wsType)
        try:
            ws = self.Spreadsheet.GetWorksheetDefinition(plantId, wsType)[0]
        except:
            return
        if not ws.columns:
            return

        if (viewName != ""):
            configsInPlant = self.Configuration.GetSpreadsheetViews(plantId)
            configsWithName = [config.configurationData for config in configsInPlant if json.loads(
                config.configurationData).get('name') == viewName]
            columnNumbersInConfig = [json.loads(jsData).get(
                'columnNumbers') for jsData in configsWithName]
            columnsInView = []
            for lst in columnNumbersInConfig:
                for item in lst:
                    columnsInView.append(item)

        columnDict = {}
        for column in ws.columns:
            if (viewName != ""):
                if(column.columnNumber in columnsInView):
                    columnDict[column.columnNumber] = [column.name,  column.columnId, column.parameterId,  column.displayUnitId,
                                                       column.lastRowNumberWithData, column.locationId, column.limits, column.dataSourceBinding.bindingId]
            else:
                columnDict[column.columnNumber] = [column.name,  column.columnId, column.parameterId,  column.displayUnitId,
                                                   column.lastRowNumberWithData, column.locationId, column.limits, column.dataSourceBinding.bindingId]
        limitDict = {}
        for column in columnDict.values():
            try:
                limit = column[6][0]
                if limit.enumLimit == 5 or limit.enumLimit == 4 or limit.enumLimit == 1:
                    limitDict[column[1]] = [limit.name, limit.lowValue.value, limit.lowOperation, limit.highValue.value,
                                            limit.highOperation,  limit.timeWindow.startTime.jsonDateTime.value, limit.timeWindow.endTime.jsonDateTime.value]

            except:
                pass

        twinDict = self.PathFinder(plantId, columnDict)
        for key in columnDict.keys():
            locationType = typeDict[twinDict[columnDict[key][5]][4]][1]
            try:
                dataSourceBinding = columnDict[key][7]
            except:
                dataSourceBinding = ""
            try:
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                lat = twinDict[columnDict[key][1]][6]
                long = twinDict[columnDict[key][1]][7]
                worksheetWriter.writerow({'ColumnNumber': key,
                                          'Name': columnDict[key][0],
                                          'ColumnId': columnDict[key][1],
                                          'Worksheet Type': wsVal,
                                          'ParameterId': columnDict[key][2],
                                          'UnitId': columnDict[key][3],
                                          'LocationName': twinDict[columnDict[key][1]][3],
                                          'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                          'UnitTranslation': unitDict[columnDict[key][3]][2],
                                          'LocationId': columnDict[key][5],
                                          'LocationType': locationType,
                                          'ParameterTranslation': paramDict[columnDict[key][2]][1],
                                          "Path": twinDict[columnDict[key][1]][2],
                                          "Latitude": lat,
                                          "Longitude": long,
                                          'LimitName':  limitDict[columnDict[key][1]][0],
                                          "LowValue": limitDict[columnDict[key][1]][1],
                                          "LowOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][2]),
                                          "HighValue": limitDict[columnDict[key][1]][3],
                                          "HighOperation": self.LimitOperationToStringValue(limitDict[columnDict[key][1]][4]),
                                          "LimitStartTime": limitDict[columnDict[key][1]][5],
                                          "LimitEndTime": limitDict[columnDict[key][1]][6],
                                          "LocationSubtype": locationSubtype,
                                          "DataBinding": dataSourceBinding
                                          })
            except (KeyError):
                locationSubtype = subtypeDict[twinDict[columnDict[key][5]][5]][1]
                lat = twinDict[columnDict[key][1]][6]
                long = twinDict[columnDict[key][1]][7]
                worksheetWriter.writerow({'ColumnNumber': key,
                                          'Name': columnDict[key][0],
                                          'ColumnId': columnDict[key][1],
                                          'Worksheet Type': wsVal,
                                          'ParameterId': columnDict[key][2],
                                          'UnitId': columnDict[key][3],
                                          'LocationName': twinDict[columnDict[key][1]][3],
                                          'LastPopulatedDate': GetDateFromRowNumber(columnDict[key][4], wsType),
                                          'UnitTranslation': unitDict[columnDict[key][3]][2],
                                          'LocationId': columnDict[key][5],
                                          'LocationType': locationType,
                                          'ParameterTranslation': paramDict[columnDict[key][2]][1],
                                          "Path": twinDict[columnDict[key][1]][2],
                                          "Latitude": lat,
                                          "Longitude": long,
                                          "LocationSubtype": locationSubtype,
                                          "DataBinding": dataSourceBinding
                                          })

    def __mapUnitsAndParams(self):
        units = self.Library.GetUnits()
        parameters = self.Library.GetParameters()
        i18N = self.Library.Geti18nKeys("AQI_FOUNDATION_LIBRARY")[
            0].get("AQI_FOUNDATION_LIBRARY")
        i18NUnits = i18N.get("UnitType").get("LONG")
        i18NParams = i18N.get("Parameter").get("LONG")
        i18NSubtypes = i18N.get("DigitalTwinSubType")
        paramDict = {}
        for param in parameters:
            try:
                paramDict[param.IntId] = [
                    param.i18nKey, i18NParams[param.i18nKey]]
            except:
                paramDict[param.IntId] = [param.i18nKey, None]
        unitDict = {}
        for unit in units:
            try:
                unitDict[unit.IntId] = [unit.i18nKey,
                                        unit.unitName, i18NUnits[unit.i18nKey]]
            except:
                unitDict[unit.IntId] = [unit.i18nKey, unit.unitName, None]
        twinTypes = self.DigitalTwin.GetDigitalTwinTypes()
        typeDict = {}
        for twinType in twinTypes:
            typeDict[twinType.id] = [
                twinType.i18NKeyName, twinType.description.value]
        twinSubtypes = self.DigitalTwin.GetDigitalTwinSubtypes()
        subTypeDict = {}
        for subtype in twinSubtypes:
            try:
                subTypeDict[subtype.id] = [subtype.i18NKeyName,
                                           i18NSubtypes[subtype.i18NKeyName]]
            except(KeyError):
                subTypeDict[subtype.id] = [
                    subtype.i18NKeyName, subtype.i18NKeyName]

        return unitDict, paramDict, typeDict, subTypeDict

    def ExportColumnDetailsByType(self, filename, plantId, wsType=4,  viewName=""):
        unitDict, paramDict, typeDict, subtypeDict = self.__mapUnitsAndParams()
        with open(filename, mode='w', newline='', encoding="utf-8") as file:
            fieldnames = self.columnFieldNames
            worksheetWriter = csv.DictWriter(file, fieldnames=fieldnames)
            worksheetWriter.writeheader()
            self.__mapAndWriteColumns(
                plantId, wsType, unitDict, paramDict, worksheetWriter, typeDict, subtypeDict, viewName="")

    def PathFinder(self, plantId, columnDict):
        twins = self.DigitalTwin.GetDescendantsByType(
            plantId, "ae018857-5f27-4fe4-8117-d0cbaecb9c1e", False)
        twins.MergeFrom(
            self.DigitalTwin.GetDescendantsByRefByCategory(plantId, 2, False))
        twins = twins.items
        twinDict = {}
        lat = ""
        long = ""
        for twin in twins:
            lat = twin.geography.point2d.y
            long = twin.geography.point2d.x
            twinDict[twin.twinReferenceId.value] = [twin.parentTwinReferenceId.value,
                                                    twin.name.value, None, None, twin.twinTypeId, twin.twinSubTypeId.value, lat, long]

        for key in columnDict.keys():
            twinId = columnDict[key][1]
            path = []
            twinChain = []
            pathString = ""
            while (twinId != plantId):
                path.append(twinDict[twinId][1])
                twinChain.append(twinId)
                twinId = twinDict[twinId][0]
            path.append(twinDict[twinId][1])
            twinChain.append(plantId)
            for twinRef in twinChain:
                if not twinDict[twinRef][6] or not twinDict[twinRef][7]:
                    lat = ""
                    long = ""
                else:
                    lat = twinDict[twinRef][6]
                    long = twinDict[twinRef][7]
                if lat and long:
                    break

            twinDict[twinChain[0]][6] = lat
            twinDict[twinChain[0]][7] = long
            twinDict[columnDict[key][1]][3] = path[1]
            while (path):
                pathString = f'{pathString}/{path.pop()}'
            twinDict[columnDict[key][1]][2] = pathString
        return twinDict

    def ConvertWSTypeToStringValue(self, wsType):
        if wsType == 1:
            return "Fifteen Minute"
        elif wsType == 2:
            return "Hourly"
        elif wsType == 3:
            return "Four Hour"
        elif wsType == 4:
            return "Daily"
        else:
            return print("Enter valid worksheet type value (1= Fifteen minute, 2=Hourly, 3=FourHour, 4=Daily)")

    def ConvertLimitEnumToStringValue(self, limitEnum):
        if limitEnum == 1:
            return "Regulatory"
        elif limitEnum == 2:
            return "Warning"
        elif limitEnum == 3:
            return "Warning"
        elif limitEnum == 4:
            return "Regulatory"
        elif limitEnum == 5:
            return "Regulatory"
        elif limitEnum == 6:
            return "Warning"
        elif limitEnum == 7:
            return "Goal"
        else:
            return print("Enter valid worksheet type value (1-7)")

    def LimitOperationToStringValue(self, limitOperation):
        if limitOperation == 1:
            return ">"
        elif limitOperation == 2:
            return ">="
        elif limitOperation == 3:
            return "<"
        elif limitOperation == 4:
            return "=<"
        elif limitOperation == 0:
            return "Undefined"
        else:
            return print(f"{limitOperation}Enter valid limit Operation value (1 is >, 2  is ≥  , 3 is < , 4 is ≤)")

    def EnumDataSourceToStringValue(self, enumDataSource):
        if enumDataSource == 1:
            return "MOBILE"
        elif enumDataSource == 2:
            return "COMPUTATION"
        elif enumDataSource == 3:
            return "WEBAPP"
        elif enumDataSource == 4:
            return "INSTRUMENT"
        elif enumDataSource == 0:
            return "UNKNOWN"
        elif enumDataSource == 5:
            return "IMPORT"
        elif enumDataSource == 6:
            return "SPREADSHEETDEF"
        elif enumDataSource == 7:
            return "CONNECT"
        else:
            return print(f"{enumDataSource}Enter valid enumDataSource (valid values are 0-7))")

    def ParseAuditTime(self, dateEntered):       
        dateEntered =datetime.strptime(dateEntered[:15], '%Y-%m-%dT%H:%M')
        dateEntered = dateEntered.replace(tzinfo=timezone.utc)        
        return dateEntered        


import requests
import json
from logging import Logger
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class ConfigurationApi:
    def __init__(self, env, auth):
        self.AppUrl = "/common/configuration/v2/"
        self.Environment = env
        self.Authentication = auth

    def GetSpreadsheetViews(self, authTwinId, configurationTypeId='bedc5ff2-bc8e-4916-9560-ccc28701d792'):
        url = f'{self.Environment}{self.AppUrl}?authTwinRefId={authTwinId}&configurationTypeId={configurationTypeId}'
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.configurations.items



from datetime import datetime, timezone, timedelta
from one_interfaces import jsonTicksDateTime_pb2 as jsonTicksTime


def GetRowNumber(date: datetime, wsType):
    if not date.tzinfo:
        date = date.replace(tzinfo=timezone.utc)
    BaseTime = datetime(1900, 1, 1, 0, 0, 0, 0, timezone.utc)
    diffTime = date - BaseTime
    windowSize = TimeSpanOfWorksheetType(wsType)
    diffTimeMinutes = diffTime.total_seconds()/60
    windowSizeMinutes = windowSize.total_seconds()/60
    return int(diffTimeMinutes / windowSizeMinutes) + 1


def GetDateFromRowNumber(rowNumber, wsType):
    BaseTime = datetime(1900, 1, 1, 0, 0, 0, 0, timezone.utc)
    row = rowNumber-1
    windowSize = TimeSpanOfWorksheetType(wsType)
    windowSizeMinutes = windowSize.total_seconds()/60
    mins = windowSizeMinutes*row
    return BaseTime+timedelta(minutes=mins)


def ToJsonTicksDateTime(date: datetime):
    jsTime = jsonTicksTime.JsonTicksDateTime()
    jsTime.jsonDateTime.value = str(date)
    return jsTime


def TimeSpanOfWorksheetType(wsType):
    if wsType == 1:
        return timedelta(minutes=15)
    elif wsType == 2:
        return timedelta(hours=1)
    elif wsType == 3:
        return timedelta(hours=4)
    elif wsType == 4:
        return timedelta(days=1)
    else:
        return "Invalid worksheet type"



from one_interfaces import apierror_pb2 as apiError
from one_interfaces import apiresponse_pb2 as apiResponse
import google
import logging


def DeserializeResponse(response):
    try:
        pbResponse = apiResponse.ApiResponse()
        pbResponse.ParseFromString(response.content)
        return pbResponse
    except Exception as Argument:
        logging.exception("Error occured in deserialization process")
        pass



import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
import uuid

class ReportApi:
    def __init__(self, env, auth: AuthenticationApi):
        self.AppUrl = "/enterprise/report/v1/"
        self.Environment = env
        self.Authentication = auth
        
    def GetReportDefinitions(self, plantId=None):
        url = f'{self.Environment}{self.AppUrl}definitions'
        if plantId:
            url= url+f"?plantId={plantId}"
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.ReportDefinitions.items
    
    def GetReportColumnsIds(self, plantId= None):        
        reportDefs = [json.loads(report.reportDefinitionJson.value) for report  in self.GetReportDefinitions(plantId)]
        columns = [d.get('columns') for d in reportDefs]
        ids =[]
        for colLst in columns:
            for col in colLst:
                ids.append(col.get('id'))
        uniqueIds =[]
        for id in ids:
            if id not in uniqueIds:
                uniqueIds.append(id)
        return uniqueIds



from datetime import datetime
import uuid
import requests
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_py_sdk.shared.helpers.datetimehelper import *
from one_interfaces import row_pb2 as row
from one_interfaces import cell_pb2 as cell   
from one_interfaces import celldata_pb2 as celldata   
from one_interfaces import auditevent_pb2 as audit
from one_interfaces import note_pb2 as note

class SpreadsheetApi:
    def __init__(self, env: str, auth: AuthenticationApi):
        self.Environment = env
        self.Auth = auth
        self.AppUrl = "/operations/spreadsheet/v1/"
    
    def GetWorksheetColumnIds(self, plantId, wsType):        
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        columnIds = [
            col.columnId for col in response.content.worksheetDefinitions.items[0].columns if col.isActive == True]
        return columnIds

    def GetWorksheetColumnNumbers(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        columnNumbers = [
            col.columnNumber for col in response.content.worksheetDefinitions.items[0].columns if col.isActive == True]
        return columnNumbers

    def GetWorksheetDefinition(self, plantId, wsType):
        url = self.Environment + self.AppUrl + plantId + \
            "/worksheet/"+str(wsType)+"/definition"
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.worksheetDefinitions.items

    def GetSpreadsheetDefinition(self, plantId):
        url = f'{self.Environment}{self.AppUrl}{plantId}/definition'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        return response.content.spreadsheetDefinitions.items

    def GetColumnByDay(self, plantId, wsType, columnId, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/byday/{date.year}/{date.month}/{date.day}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetColumnByMonth(self, plantId: str, wsType: int, columnId: int, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/bymonth/{date.year}/{date.month}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetColumnByYear(self, plantId, wsType, columnId, date: datetime):
        url = self.Environment + self.AppUrl + plantId + \
            f'/worksheet/{str(wsType)}/column/{columnId}/byyear/{date.year}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.measurements.items

    def GetRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url = url+f'&startRow={startRow}'
        if endRow:
            url = url+f'&endRow={endRow}'
        if columns:
            url = url+f'&columns={columns}'
        if viewId:
            url = url+f'&viewId={viewId}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.rows.items

    def __getRows(self, plantId, wsType, startRow=None, endRow=None, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        requestId = uuid.uuid4()
        url = f'{self.Environment}{self.AppUrl}{plantId}/worksheet/{str(wsType)}/rows?requestId={requestId}'
        if startRow:
            url = url+f'&startRow={startRow}'
        if endRow:
            url = url+f'&endRow={endRow}'
        if columns:
            url = url+f'&columns={columns}'
        if viewId:
            url = url+f'&viewId={viewId}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response        
        return response.content.rows

    def GetRowsByDay(self, plantId, wsType, date: datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl + \
            f'{plantId}/worksheet/{str(wsType)}/rows/byday/{date.year}/{date.month}/{date.day}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.rows.items

    def GetRowsByMonth(self, plantId, wsType, date: datetime, columns=None, viewId=None):
        if columns and viewId:
            return print("Using both columns and viewId parameters together is not supported.")
        url = self.Environment + self.AppUrl + \
            f'{plantId}/worksheet/{str(wsType)}/rows/bymonth/{date.year}/{date.month}'
        headers = {'Authorization': self.Auth.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.rows.items

    def GetRowsForTimeRange(self, plantId, wsType, startDate: datetime, endDate: datetime):
        startRow = GetRowNumber(startDate, wsType)
        endRow = GetRowNumber(endDate, wsType)
        rows = row.Rows()
        while endRow - startRow > 5000:
            newEndRow = startRow+5000
            rows.MergeFrom(self.__getRows(
                plantId, wsType, startRow, newEndRow))
            startRow = newEndRow+1
        rows.MergeFrom(self.__getRows(plantId, wsType, startRow, endRow))
        return rows.items



import requests
import json
from datetime import datetime, time, timedelta
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse
from one_interfaces import user_pb2 as User
from one_interfaces import role_pb2 as role

import google


class AuthenticationApi:
    def __init__(self, env):
        self.Environment = env
        self.Token = Token()
        self.UserName = ""
        self.Password = ""
        self.User: User = User.User()
        self.IsAuthenticated = False

    def GetToken(self, user, password):
        data = {'username': user, 'password': password, 'grant_type': 'password', 'scope': 'FFAccessAPI openid',
                'client_id': 'VSTestClient', 'client_secret': '0CCBB786-9412-4088-BC16-78D3A10158B7'}
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/x-www-form-urlencoded'}
        url = self.Environment+"/connect/token"
        response = requests.post(url, headers=headers, data=data)
        if (response.status_code != 200):
            return ''
        self.__setToken(response)
        self.IsAuthenticated = True
        self.UserName = user
        self.Password = password
        return self.Token.access_token

    def GetUserInfo(self):
        headers = {'Accept': 'application/json',
                   "Authorization": self.Token.access_token}
        url = self.Environment+"/connect/userinfo"
        response = requests.get(url, headers=headers)
        self.__setInfo(response)
        return response.content

    def LoginResourceOwner(self, userName, password):
        data = {'username': userName, 'password': password, 'grant_type': 'password', 'scope': 'FFAccessAPI openid',
                'client_id': 'VSTestClient', 'client_secret': '0CCBB786-9412-4088-BC16-78D3A10158B7'}
        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/x-www-form-urlencoded'}
        url = self.Environment+"/connect/token"
        response = requests.post(url, headers=headers, data=data)
        if (response.status_code != 200):
            return False
        self.__setToken(response)
        self.IsAuthenticated = True
        self.UserName = userName
        self.Password = password
        return True

    def Logout(self):
        headers = {'Accept': 'application/json',
                   "Authorization": self.Token.access_token}
        url = self.Environment+"/account/logout"
        response = requests.post(url, headers=headers)
        self.Token = Token()
        self.UserName = ""
        self.Password = ""
        self.User: User = User.User()
        self.IsAuthenticated = False

    def __setInfo(self, response):
        jResponse = json.loads(response.content)
        self.User.firstName.value = jResponse.get('given_name')
        self.User.lastName.value = jResponse.get('family_name')
        self.User.userName = jResponse.get('user_name')
        self.User.email.value = jResponse.get('email')
        self.User.tenantId = jResponse.get('ActiveTenantId')
        self.User.id = jResponse.get('sub')

    def __setToken(self, tokenResponse):
        token = Token()
        token.created = datetime.now()
        responseJson = json.loads(tokenResponse.content)
        self.IsAuthenticated = True
        token.token_type = responseJson['token_type']
        token.scope = responseJson['scope']
        token.access_token = token.token_type + \
            " " + responseJson['access_token']
        token.expires_in = token.created + \
            timedelta(seconds=responseJson['expires_in'])
        self.Token = token


class Token:
    def __init__(self):
        self.access_token: str
        self.expires_in: float
        self.token_type: str
        self.scope: str
        self.created: datetime

    def __repr__(self):
        return "Access token: %s, Created on: %s, Expires in: %s, Token Type: %s, Scope: %s  " % (self.access_token, self.created, self.expires_in, self.token_type, self.scope)



from datetime import datetime
import requests
import json
from one_py_sdk.enterprise.authentication import AuthenticationApi
from one_py_sdk.shared.helpers.protobufhelper import DeserializeResponse


class HistorianApi:
    def __init__(self, env, auth: AuthenticationApi):
        self.Environment = env
        self.Authentication = auth
        self.AppUrl = "/historian/data/v1/"

    def GetHistorianData(self, twinRefId, date: datetime):
        date = f'{str(date.date())}T{str(date.time())}Z'
        url = f'{self.Environment}{self.AppUrl}{twinRefId}/{date}'
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.historianDatas.items

    def GetHistorianDataRange(self, twinRefId, startTime: datetime, endTime: datetime):
        startString = f'{str(startTime.date())}T{str(startTime.time())}'
        endString = f'{str(endTime.date())}T{str(endTime.time())}'
        url = f'{self.Environment}{self.AppUrl}{twinRefId}?startTime={startString}&endTime={endString}'
        headers = {'Authorization': self.Authentication.Token.access_token,
                   "Accept": "application/x-protobuf"}
        response = DeserializeResponse(requests.get(url, headers=headers))
        if response.errors:
            return response
        return response.content.historianDatas.items
